from typing import Optional

from pydantic import Field

from ome_types._autogenerated.ome_2016_06.settings import Settings
from ome_types._autogenerated.ome_2016_06.units_length import UnitsLength

__NAMESPACE__ = "http://www.openmicroscopy.org/Schemas/OME/2016-06"


class LightSourceSettings(Settings):
    """
    Attributes
    ----------
    id : str
        (The LightSourceSettings ID).
    attenuation : None | float
        The Attenuation of the light source [units:none] A fraction, as a value
        from 0.0 to 1.0.
    wavelength : None | float
        The Wavelength of the light source. Units are set by WavelengthUnit.
    wavelength_unit : UnitsLength
        The units of the Wavelength of the light source - default:nanometres[nm]
    """

    class Meta:
        namespace = "http://www.openmicroscopy.org/Schemas/OME/2016-06"

    id: str = Field(
        default="__auto_sequence__",
        pattern=r"(urn:lsid:([\w\-\.]+\.[\w\-\.]+)+:LightSource:\S+)|(LightSource:\S+)",
        json_schema_extra={
            "name": "ID",
            "type": "Attribute",
            "required": True,
            "pattern": r"(urn:lsid:([\w\-\.]+\.[\w\-\.]+)+:LightSource:\S+)|(LightSource:\S+)",
        },
    )
    attenuation: Optional[float] = Field(
        default=None,
        ge=0.0,
        le=1.0,
        json_schema_extra={
            "name": "Attenuation",
            "type": "Attribute",
            "min_inclusive": 0.0,
            "max_inclusive": 1.0,
        },
    )
    wavelength: Optional[float] = Field(
        default=None,
        gt=0.0,
        json_schema_extra={
            "name": "Wavelength",
            "type": "Attribute",
            "min_exclusive": 0.0,
        },
    )
    wavelength_unit: UnitsLength = Field(
        default=UnitsLength.NANOMETER,
        json_schema_extra={
            "name": "WavelengthUnit",
            "type": "Attribute",
        },
    )
