from typing import Optional

from pydantic import Field

from ome_types._autogenerated.ome_2016_06.marker import Marker
from ome_types._autogenerated.ome_2016_06.shape import Shape

__NAMESPACE__ = "http://www.openmicroscopy.org/Schemas/OME/2016-06"


class Line(Shape):
    """
    A straight line defined by it's end points.

    Attributes
    ----------
    x1 : float
        The X coordinate of the start of the line. [units pixels]
    y1 : float
        The Y coordinate of the start of the line. [units pixels]
    x2 : float
        The X coordinate of the end of the line. [units pixels]
    y2 : float
        The Y coordinate of the end of the line. [units pixels]
    marker_start : None | Marker
        (The Line MarkerStart).
    marker_end : None | Marker
        (The Line MarkerEnd).
    """

    class Meta:
        namespace = "http://www.openmicroscopy.org/Schemas/OME/2016-06"

    x1: float = Field(
        json_schema_extra={
            "name": "X1",
            "type": "Attribute",
            "required": True,
        }
    )
    y1: float = Field(
        json_schema_extra={
            "name": "Y1",
            "type": "Attribute",
            "required": True,
        }
    )
    x2: float = Field(
        json_schema_extra={
            "name": "X2",
            "type": "Attribute",
            "required": True,
        }
    )
    y2: float = Field(
        json_schema_extra={
            "name": "Y2",
            "type": "Attribute",
            "required": True,
        }
    )
    marker_start: Optional[Marker] = Field(
        default=None,
        json_schema_extra={
            "name": "MarkerStart",
            "type": "Attribute",
        },
    )
    marker_end: Optional[Marker] = Field(
        default=None,
        json_schema_extra={
            "name": "MarkerEnd",
            "type": "Attribute",
        },
    )
