from typing import Optional

from pydantic import Field

from ome_types._autogenerated.ome_2016_06.annotation_ref import AnnotationRef
from ome_types._autogenerated.ome_2016_06.units_length import UnitsLength
from ome_types._autogenerated.ome_2016_06.units_time import UnitsTime
from ome_types._mixins._base_type import OMEType

__NAMESPACE__ = "http://www.openmicroscopy.org/Schemas/OME/2016-06"


class Plane(OMEType):
    """
    The Plane object holds microscope stage and image timing data for a given
    channel/z-section/timepoint.

    Attributes
    ----------
    hash_sha1 : None | bytes
        (The Plane HashSHA1).
    annotation_refs : list[AnnotationRef]
        (The Plane AnnotationRefs).
    the_z : int
        The Z-section this plane is for. [units:none] This is numbered from 0.
    the_t : int
        The timepoint this plane is for. [units:none] This is numbered from 0.
    the_c : int
        The channel this plane is for. [units:none] This is numbered from 0.
    delta_t : None | float
        Time since the beginning of the experiment. Units are set by DeltaTUnit.
    delta_t_unit : UnitsTime
        The units of the DeltaT - default:seconds[s].
    exposure_time : None | float
        The length of the exposure. Units are set by ExposureTimeUnit.
    exposure_time_unit : UnitsTime
        The units of the ExposureTime - default:seconds[s].
    position_x : None | float
        The X position of the stage. Units are set by PositionXUnit.
    position_x_unit : UnitsLength
        The units of the X stage position - default:[reference frame].
    position_y : None | float
        The Y position of the stage. Units are set by PositionYUnit.
    position_y_unit : UnitsLength
        The units of the Y stage position - default:[reference frame].
    position_z : None | float
        The Z position of the stage. Units are set by PositionZUnit.
    position_z_unit : UnitsLength
        The units of the Z stage position - default:[reference frame].
    """

    class Meta:
        namespace = "http://www.openmicroscopy.org/Schemas/OME/2016-06"

    hash_sha1: Optional[bytes] = Field(
        default=None,
        json_schema_extra={
            "name": "HashSHA1",
            "type": "Element",
            "length": 20,
            "format": "base16",
        },
    )
    annotation_refs: list[AnnotationRef] = Field(
        default_factory=list,
        json_schema_extra={
            "name": "AnnotationRef",
            "type": "Element",
        },
    )
    the_z: int = Field(
        ge=0,
        json_schema_extra={
            "name": "TheZ",
            "type": "Attribute",
            "required": True,
            "min_inclusive": 0,
        },
    )
    the_t: int = Field(
        ge=0,
        json_schema_extra={
            "name": "TheT",
            "type": "Attribute",
            "required": True,
            "min_inclusive": 0,
        },
    )
    the_c: int = Field(
        ge=0,
        json_schema_extra={
            "name": "TheC",
            "type": "Attribute",
            "required": True,
            "min_inclusive": 0,
        },
    )
    delta_t: Optional[float] = Field(
        default=None,
        json_schema_extra={
            "name": "DeltaT",
            "type": "Attribute",
        },
    )
    delta_t_unit: UnitsTime = Field(
        default=UnitsTime.SECOND,
        json_schema_extra={
            "name": "DeltaTUnit",
            "type": "Attribute",
        },
    )
    exposure_time: Optional[float] = Field(
        default=None,
        json_schema_extra={
            "name": "ExposureTime",
            "type": "Attribute",
        },
    )
    exposure_time_unit: UnitsTime = Field(
        default=UnitsTime.SECOND,
        json_schema_extra={
            "name": "ExposureTimeUnit",
            "type": "Attribute",
        },
    )
    position_x: Optional[float] = Field(
        default=None,
        json_schema_extra={
            "name": "PositionX",
            "type": "Attribute",
        },
    )
    position_x_unit: UnitsLength = Field(
        default=UnitsLength.REFERENCEFRAME,
        json_schema_extra={
            "name": "PositionXUnit",
            "type": "Attribute",
        },
    )
    position_y: Optional[float] = Field(
        default=None,
        json_schema_extra={
            "name": "PositionY",
            "type": "Attribute",
        },
    )
    position_y_unit: UnitsLength = Field(
        default=UnitsLength.REFERENCEFRAME,
        json_schema_extra={
            "name": "PositionYUnit",
            "type": "Attribute",
        },
    )
    position_z: Optional[float] = Field(
        default=None,
        json_schema_extra={
            "name": "PositionZ",
            "type": "Attribute",
        },
    )
    position_z_unit: UnitsLength = Field(
        default=UnitsLength.REFERENCEFRAME,
        json_schema_extra={
            "name": "PositionZUnit",
            "type": "Attribute",
        },
    )
