from typing import Optional

from pydantic import Field

from ome_types._autogenerated.ome_2016_06.affine_transform import (
    AffineTransform,
)
from ome_types._autogenerated.ome_2016_06.annotation_ref import AnnotationRef
from ome_types._autogenerated.ome_2016_06.shape_fill_rule import Shape_FillRule
from ome_types._autogenerated.ome_2016_06.shape_font_family import (
    Shape_FontFamily,
)
from ome_types._autogenerated.ome_2016_06.shape_font_style import (
    Shape_FontStyle,
)
from ome_types._autogenerated.ome_2016_06.units_length import UnitsLength
from ome_types._mixins._base_type import OMEType
from ome_types._mixins._kinded import KindMixin
from ome_types.model._color import Color

__NAMESPACE__ = "http://www.openmicroscopy.org/Schemas/OME/2016-06"


class Shape(KindMixin, OMEType):
    """The shape element contains a single specific ROI shape and links that to any
    channels, and a timepoint and a z-section.

    It also records any transform applied to the ROI shape.

    Attributes
    ----------
    transform : None | AffineTransform
        This is a matrix used to transform the shape. The element has 6 xsd:float
        attributes. If the element is present then all 6 values must be included.
    annotation_refs : list[AnnotationRef]
        (The Shape AnnotationRefs).
    fill_color : None | Color
        The color of the fill - encoded as RGBA The value "-1" is #FFFFFFFF so
        solid white (it is a signed 32 bit value) NOTE: Prior to the 2012-06 schema
        the default value was incorrect and produced a transparent red not solid
        white.
    fill_rule : None | Shape_FillRule
        The rule used to decide which parts of the shape to fill. [enumeration]
    stroke_color : None | Color
        The color of the stroke  - encoded as RGBA The value "-1" is #FFFFFFFF so
        solid white (it is a signed 32 bit value) NOTE: Prior to the 2012-06 schema
        the default value was incorrect and produced a transparent red not solid
        white.
    stroke_width : None | float
        The width of the stroke. Units are set by StrokeWidthUnit.
    stroke_width_unit : UnitsLength
        The units used for the stroke width.
    stroke_dash_array : None | str
        e.g. "none", "10 20 30 10"
    text : None | str
        (The Shape Text).
    font_family : None | Shape_FontFamily
        The font family used to draw the text. [enumeration] Note: these values are
        all lower case so they match the standard HTML/CSS values. "fantasy" has
        been included for completeness; we do not recommend its regular use. This
        attribute is under consideration for removal from the OME-XML schema.
    font_size : None | int
        Size of the font. Units are set by FontSizeUnit.
    font_size_unit : UnitsLength
        The units used for the font size.
    font_style : None | Shape_FontStyle
        The style and weight applied to the text.  [enumeration] This is a
        simplified combination of the HTML/CSS attributes font-style AND font-
        weight.
    locked : None | bool
        Controls whether the shape is locked and read only, true is locked, false
        is editable.
    id : str
        (The Shape ID).
    the_z : None | int
        The z-section the ROI applies to. If not specified then the ROI applies to
        all the z-sections of the image. [units:none] This is numbered from 0.
    the_t : None | int
        The timepoint the ROI applies to. If not specified then the ROI applies to
        all the timepoints of the image. [units:none] This is numbered from 0.
    the_c : None | int
        The channel the ROI applies to. If not specified then the ROI applies to
        all the channels of the image. [units:none] This is numbered from 0.
    """

    transform: Optional[AffineTransform] = Field(
        default=None,
        json_schema_extra={
            "name": "Transform",
            "type": "Element",
            "namespace": "http://www.openmicroscopy.org/Schemas/OME/2016-06",
        },
    )
    annotation_refs: list[AnnotationRef] = Field(
        default_factory=list,
        json_schema_extra={
            "name": "AnnotationRef",
            "type": "Element",
            "namespace": "http://www.openmicroscopy.org/Schemas/OME/2016-06",
        },
    )
    fill_color: Optional[Color] = Field(
        default=None,
        json_schema_extra={
            "name": "FillColor",
            "type": "Attribute",
        },
    )
    fill_rule: Optional[Shape_FillRule] = Field(
        default=None,
        json_schema_extra={
            "name": "FillRule",
            "type": "Attribute",
        },
    )
    stroke_color: Optional[Color] = Field(
        default=None,
        json_schema_extra={
            "name": "StrokeColor",
            "type": "Attribute",
        },
    )
    stroke_width: Optional[float] = Field(
        default=None,
        json_schema_extra={
            "name": "StrokeWidth",
            "type": "Attribute",
        },
    )
    stroke_width_unit: UnitsLength = Field(
        default=UnitsLength.PIXEL,
        json_schema_extra={
            "name": "StrokeWidthUnit",
            "type": "Attribute",
        },
    )
    stroke_dash_array: Optional[str] = Field(
        default=None,
        json_schema_extra={
            "name": "StrokeDashArray",
            "type": "Attribute",
        },
    )
    text: Optional[str] = Field(
        default=None,
        json_schema_extra={
            "name": "Text",
            "type": "Attribute",
        },
    )
    font_family: Optional[Shape_FontFamily] = Field(
        default=None,
        json_schema_extra={
            "name": "FontFamily",
            "type": "Attribute",
        },
    )
    font_size: Optional[int] = Field(
        default=None,
        ge=0,
        json_schema_extra={
            "name": "FontSize",
            "type": "Attribute",
            "min_inclusive": 0,
        },
    )
    font_size_unit: UnitsLength = Field(
        default=UnitsLength.POINT,
        json_schema_extra={
            "name": "FontSizeUnit",
            "type": "Attribute",
        },
    )
    font_style: Optional[Shape_FontStyle] = Field(
        default=None,
        json_schema_extra={
            "name": "FontStyle",
            "type": "Attribute",
        },
    )
    locked: Optional[bool] = Field(
        default=None,
        json_schema_extra={
            "name": "Locked",
            "type": "Attribute",
        },
    )
    id: str = Field(
        default="__auto_sequence__",
        pattern=r"(urn:lsid:([\w\-\.]+\.[\w\-\.]+)+:Shape:\S+)|(Shape:\S+)",
        json_schema_extra={
            "name": "ID",
            "type": "Attribute",
            "required": True,
            "pattern": r"(urn:lsid:([\w\-\.]+\.[\w\-\.]+)+:Shape:\S+)|(Shape:\S+)",
        },
    )
    the_z: Optional[int] = Field(
        default=None,
        ge=0,
        json_schema_extra={
            "name": "TheZ",
            "type": "Attribute",
            "min_inclusive": 0,
        },
    )
    the_t: Optional[int] = Field(
        default=None,
        ge=0,
        json_schema_extra={
            "name": "TheT",
            "type": "Attribute",
            "min_inclusive": 0,
        },
    )
    the_c: Optional[int] = Field(
        default=None,
        ge=0,
        json_schema_extra={
            "name": "TheC",
            "type": "Attribute",
            "min_inclusive": 0,
        },
    )


FillRule = Shape_FillRule
FontFamily = Shape_FontFamily
FontStyle = Shape_FontStyle
