from pydantic import Field, field_validator

from ome_types._autogenerated.ome_2016_06.text_annotation import TextAnnotation
from ome_types._mixins._base_type import OMEType
from ome_types._mixins._validators import (
    any_elements_validator,
    xml_value_validator,
)

__NAMESPACE__ = "http://www.openmicroscopy.org/Schemas/OME/2016-06"


class XMLAnnotation(TextAnnotation):
    """An general xml annotation.

    The contents of this is not processed as OME XML but should still be
    well-formed XML.
    """

    class Meta:
        namespace = "http://www.openmicroscopy.org/Schemas/OME/2016-06"

    value: "XMLAnnotation.Value" = Field(
        json_schema_extra={
            "name": "Value",
            "type": "Element",
            "required": True,
        }
    )

    class Value(OMEType):
        any_elements: list[object] = Field(
            default_factory=list,
            json_schema_extra={
                "type": "Wildcard",
                "namespace": "##any",
            },
        )

        _vany = field_validator("any_elements")(any_elements_validator)

    _vval = field_validator("value", mode="before")(xml_value_validator)


Value = XMLAnnotation.Value
